import math
import random
from math import radians, degrees, atan2
from random import randrange, randint

import pyglet
import pymunk as pm
from pyglet.window import key
from pyglet.gl import *
from pymunk.vec2d import Vec2d

import sprite
import helper



class Cat(sprite.PmSprite):
    
    def __init__(self, img, window, space, lawn_mower, *args, **kwargs):
        sprite.PmSprite.__init__(self, img, window, space, *args, **kwargs)
        
        self.state = "explore"
        
        self.target_vec2d = Vec2d(0,0)
        self.lawn_mower = lawn_mower
        self.chain = None
                
        self.radius = 15
        self.mass = 3
        self.inertia = pm.moment_for_circle(self.mass, self.radius, 0, Vec2d(0,0))
        self.move_speed = 20
        self.steer_speed = 2
        
        self.body = pm.Body(self.mass, self.inertia)
        self.position = self.body.position
        self.pm_shape = pm.Circle(self.body, self.radius, Vec2d(0,0))
        self.pm_shape.friction = 0.2

    
        
        
    def update(self,dt):
        if self.state == "explore":
            self.explore(dt)
            
        if self.state == "caught":
            self.color = (100,100,255)
        
        #self.body.angle = radians(self.get_angle_to(self.window.mpos))
        self.position = self.body.position
        self.rotation = -degrees(self.body.angle)
        
        #print degrees(self.body.angle)
        
    def explore(self,dt):
        if abs(self.body.position-self.target_vec2d).get_length() < 15:
            self.body.position = self.target_vec2d
        if not self.collide(self.target_vec2d):
            if self.body.angle != radians(self.get_angle_to(self.target_vec2d)):
                self.body.angle = radians(self.get_angle_to(self.target_vec2d))
            f = self.body._bodycontents.rot * self.move_speed
            self.body.apply_impulse(Vec2d(f.x, f.y) , (0,0))
            if self.collide(self.target_vec2d):
                self.set_new_target()
        if self.collide(self.target_vec2d):
            self.set_new_target()
            
    def chain_to(self, other):
        self.chain = pm.PivotJoint(self.body, other.body, other.body.position)
        self.space.add(self.chain)
            
    def set_new_target(self):
        self.body.reset_forces()
        self.target_vec2d = self.get_random_target()
        self.target_angle = radians(self.get_angle_to(self.target_vec2d))
        #print self.target_vec2d, degrees(self.target_angle)
    
    def get_angle_to(self,other):
        xs = other.x - self.body.position.x
        ys = other.y - self.body.position.y
        theta = atan2(ys,xs)
        deg = degrees(theta)
        if deg < 0:
            deg += 360
        return deg    
    
    def get_random_target(self):
        width, height = self.window.get_size()
        x = randint(50,width-50)
        y = randint(30,height-30)
        return Vec2d(x,y)
    