import math
from math import radians, degrees

import pyglet
import pymunk as pm
from pyglet.window import key
from pymunk.vec2d import Vec2d


import sprite
import helper

class LawnMower(sprite.PmSprite):
    
    def __init__(self, img, window, space, cats, *args, **kwargs):
        sprite.PmSprite.__init__(self, img, window, space, *args, **kwargs)
        
        self.state = "alive"
        self.cats = cats
        self.cats_caught = []
        
        verts = helper.verts_img(self.image)
        self.mass = 3
        self.inertia = pm.moment_for_poly(self.mass, verts, Vec2d(0,0))
        self.move_speed = 80
        self.steer_speed = 2.5
        
        self.body = pm.Body(self.mass, self.inertia)
        self.pm_shape = pm.Poly(self.body, verts, Vec2d(0,0))
        self.pm_shape.friction = 0.3
        
    def update(self,dt):
        if self.state == "alive":
            self.move(dt)
            cat_caught = self.is_cat_caught()
            if cat_caught:
                #print 'yes'
                self.attach_cat(cat_caught)                
        
        self.position = self.body.position
        self.rotation = degrees(self.body.angle)
      
    def move(self,dt):
        if self.window.keys[key.UP]:
            f = self.body._bodycontents.rot * self.move_speed
            self.body.apply_impulse(Vec2d(f.x, -f.y) , (0,0))
        elif self.window.keys[key.DOWN]:
            f = -(self.body._bodycontents.rot * self.move_speed)
            self.body.apply_impulse(Vec2d(f.x, -f.y) , (0,0))
            
        if self.window.keys[key.RIGHT]:
            self.body.angle += radians(self.steer_speed)
        elif self.window.keys[key.LEFT]:
            self.body.angle -= radians(self.steer_speed)
            
    def is_cat_caught(self):
        for arb in self.space.arbiters:
            if (arb.a == self.pm_shape) or (arb.b == self.pm_shape):
                if arb.a == self.pm_shape:
                    for c in self.cats:
                        if c.pm_shape == arb.b:
                            if c.state != "caught":
                                return c
                elif arb.b == self.pm_shape:
                    for c in self.cats:
                        if c.pm_shape == arb.a:
                            if c.state != "caught":
                                return c
        return False
    
    def attach_cat(self,cat):
        if self.cats_caught == []:
            cat.state = "caught"
            self.cats_caught.append(cat)
            
            rot = self.body._bodycontents.rot
            offset = Vec2d(rot.x,-rot.y) * 50
            pos = self.body.position -  offset
            cat.body.position = pos
            
            cat.chain_to(self)
            
        elif self.cats_caught:
            cat.state = "caught"
            
            last_cat = self.cats_caught.pop()
            self.cats_caught.append(cat)
            
            rot = last_cat.body._bodycontents.rot
            offset = Vec2d(rot.x,-rot.y) * 50
            pos = last_cat.body.position -  offset
            cat.body.position = pos
            
            cat.chain_to(last_cat)
   

