import pyglet
import random
from random import randint

import pymunk as pm
from pyglet.gl import *
from pymunk.vec2d import Vec2d

import sprite
from res import Images
from lawnmower import LawnMower
from cat import Cat


class BaseState(object):
    "Base for all state"
    
    def __init__(self,window):
        self.window = window
        self.batches = []
        self.default_batch = pyglet.graphics.Batch()
        self.batches.append(self.default_batch)
        glClearColor(0.0,1.0,0.4,1.0)

    def on_draw(self):
        self.window.clear()
        for batch in self.batches:
            batch.draw()
    
    def update(self,dt):
        pass
    
class MenuState(BaseState):
    "Start here please"
    
    def __init__(self,window):
        BaseState.__init__(self, window)
        self.label = pyglet.text.Label("Menu")
        self.label.x, self.label.y = 300,300
        self.label.batch = self.default_batch
        self.label.color = (255,255,255,255)
        
class GameState(BaseState):
    "Gameplay here"
    
    def __init__(self,window):
        BaseState.__init__(self, window)
        
        self.sprite_mgr = sprite.SpriteManager()
        self.batches.append(self.sprite_mgr.batch)
        self.cats = []
        
        self.space = pm.Space()
        self.space.damping = 0.05
        
        self.lawn_mower = LawnMower(Images.lawn_mower, self.window, self.space, self.cats)
        self.lawn_mower.batch = self.default_batch
        self.lawn_mower.body.position = (100,100)
        self.sprite_mgr.add(self.lawn_mower)
        self.space.add(self.lawn_mower.body, self.lawn_mower.pm_shape)
        
        
        for c in range(randint(5,8)):
            c = Cat(Images.cat, self.window, self.space, self.lawn_mower)
            c.body.position = c.get_random_target()
            c.set_new_target()
            self.cats.append(c)
            self.sprite_mgr.add(c)
            self.space.add(c.body, c.pm_shape)
    
   #     for i in range(len(self.cats)):
   #         self.cats[i].state = "caught"
     #       if i < 3:
    #            self.cats[i-1].chain_to(self.cats[i])
                
    #    joint = pm.PivotJoint(self.cats[0].body, self.lawn_mower.body, self.lawn_mower.body.position)
    #    self.space.add(joint)
        
    
    def update(self,dt):
        self.sprite_mgr.update(dt)
        self.space.step(dt)
        